///////////////////////////////////////////////////////////////////////////////////////////////////////////////
//=============================================================================================================
// INTRODUCTION
//=============================================================================================================
//  Project/Example: mClock (X02) 
//  Purpose: This project uses the Real Time Clock (RTC) module to keep track of time. The time is retrieved 
//            from the RTC module and displayed on an OLED module. This project is an adaptation of the MWatch 
//            project from the first generation mCookie Maker Kits.
//  License: GNU General Public License v3
//  Link: https://wiki.microduino.cc/index.php/MClock_(X02)
// 
//  Author(s): Microduino, Sonny Sung
//  E-mail(s): support at microduino.cc, sonny at microduino.cc
//
//  Version: 1.0
//  Date: ??/??/??
//  Notes: Initial version
//
//  Version: 1.1
//  Date: 2017/04/20
//  Notes: Updated for the X02 Series kits.
//=============================================================================================================
// REQUIRED HARDWARE
//=============================================================================================================
//  COMPATIBLE KITS
//    [ ] 101 Basic Kit
//    [ ] 201 Advanced Kit
//    [ ] 301 Expert Kit
//    [ ] 102 Basic Kit
//    [*] 202 Advanced Kit
//    [*] 302 Expert Kit
//    [ ] Itty Bitty City Kit
//=============================================================================================================
//  MODULES
//    [1] Core, Core+, CoreUSB -OR- CoreRF
//    [1] mBattery
//    [1] Real Time Clock (RTC) Module
//    [1] OLED Module
//  SENSORS/TRINKETS
//    -
//  OTHER
//    [1] MicroUSB Cable
//=============================================================================================================
// SETUP
//=============================================================================================================
//  1. Stack all three of the modules, mCookie Core, mCookie RTC, and mCookie OLED onto mBattery. Ensure that 
//      mCookie OLED is on top of the stack.
//  2. Plug in the MicroUSB cable to the mBattery to a computer.
//  3. Visit the "TWEAKING" section on how to configure the date / time.
//  4. Program the example by pressing "upload".
//=============================================================================================================
// USAGE
//=============================================================================================================
//  Turn on the project and the time stored on the Real Time Clock (RTC) module will be retrieved and displayed
//  on the OLED screen. Powering off the project will maintain the time since the Real Time Clock (RTC) has a 
//  capacitor that enables keeping track of the time even when the power source is off.
//=============================================================================================================
// TWEAKING
//=============================================================================================================
//  "userDef.h" tab has configurable variables.
//    SET_TIME
//      Allows the time to be set on the Real Time Module.
//      Set to "true" to set the date / time on power up.
//      Set to "false" to disable setting time on power up.
//      Recommeneded to set to false and re-upload once time has been set.
//    DATE_YEAR
//      Sets the last two digits of the year. Year: 20__
//    DATE_MONTH 
//      Sets the month. Month: Jan=1, Feb=2...Dec=12
//    DATE_DAY s
//      Sts the day in the month
//    DATE_WEEKDAY
//      Sets the weekday value. Weekday: Sun=0, Mon=1...Sat=6
//    TIME_HOUR
//      Sets the hour.
//    TIME_MINUTE
//      Sets the minute.
//    TIME_SECOND
//      Sets the second.
//    STRING_MESSAGE s
//      Sets the scrolling message on display. Long messages will overlap.
//
//  Visit https://wiki.microduino.cc/index.php/MClock_(X02)#Tweaking for further tweaking instructions.
//=============================================================================================================
// NOTES
//=============================================================================================================
//  "My time resets each time I power on mClock!"
//    *Configure the date / time on userDef.h with the SET_TIME = true. Then upload.
//    *After upload is complete. Change SET_TIME = false. Re-upload. This will disable setting the time on 
//      power up.
//
//  "I am having problems uploading sketches / examples that use the RTC module!"
//    Visit: https://wiki.microduino.cc/index.php/MClock_(X02)#Troubleshooting
//
//  Additional info at: https://wiki.microduino.cc/index.php/MClock_(X02)
///////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include "userDef.h"
#include "Rtc_Pcf8563.h"

Rtc_Pcf8563 rtc;

#include "Wire.h"

#include "U8glib.h"
//Defines the different font sizes for (S)mall, (M)edium, (L)arge
#define setFont_L u8g.setFont(u8g_font_7x13)
#define setFont_M u8g.setFont(u8g_font_fixed_v0r)
#define setFont_S u8g.setFont(u8g_font_fixed_v0r)

#define setFont_SS u8g.setFont(u8g_font_fub25n)
/*
font:
 u8g_font_7x13
 u8g_font_fixed_v0r
 u8g_font_chikitar
 u8g_font_osb21
 u8g_font_courB14r
 u8g_font_courB24n
 u8g_font_9x18Br
 */
//OLED module construction for mCookie OLED
U8GLIB_SSD1306_128X64 u8g(U8G_I2C_OPT_NONE);

#define init_draw 10	//wait some milliseconds before drawing onto screen

unsigned long timer_draw,timer;
int pkj=0;
String dateStr, ret;

void setup() {

  Serial.begin(9600);

  //This line sets the time on the Real Time Clock module
  //Once set, the RTC module will store the time even when powered off.
  //Comment the "setRtcTime" with "//" to disable this once the time has been set.
  //Or else the time will be set everytime the Core module starts up.
  //setRtcTime(year, month, day, week, hour, minute, sec);
  if(SET_TIME){
    setRtcTime(DATE_YEAR, DATE_MONTH, DATE_DAY, DATE_WEEKDAY, TIME_HOUR, TIME_MINUTE, TIME_SECOND);
  }
  timer=millis();
}

void loop() {
  ret = getRtcTimeString();

  if (millis() - timer_draw > init_draw)
  {
    pkj-=4;
    u8g.firstPage();
    do {
      draw();
    }
    while ( u8g.nextPage() );
    timer_draw = millis();
  }
}

//Draws output to the OLED screen
void draw()
{
  setFont_L;
  u8g.setPrintPos(4, 16);
  u8g.print(rtc.formatDate());
  u8g.print("    ");
  switch (rtc.getWeekday()) {
    case 1:
      u8g.print("Mon");
      break;
    case 2:
      u8g.print("Tue");
      break;
    case 3:
      u8g.print("Wed");
      break;
    case 4:
      u8g.print("Thu");
      break;
    case 5:
      u8g.print("Fri");
      break;
    case 6:
      u8g.print("Sat");
      break;
    case 7:
      u8g.print("Sun");
      break;
  }
  u8g.setPrintPos(pkj, 64); 
  u8g.print(STRING_MESSAGE);
  setFont_SS;
  u8g.setPrintPos(18, 49);
  u8g.print(rtc.getHour());
  u8g.setPrintPos(55, 46);
  if (rtc.getSecond() % 2 == 0)
    u8g.print(":");
  else
    u8g.print(" ");
  u8g.setPrintPos(68, 48);
  if (rtc.getMinute() < 10)
  {
    u8g.print("0");
    u8g.print(rtc.getMinute());
  }
  else
    u8g.print(rtc.getMinute());
}

//This function sets the time on the Real Time Clock module
//Once set, the the RTC module will store the value until overwritten.
void setRtcTime (byte _year, byte _month, byte _day, byte _week, byte _hour, byte _minute, byte _sec)
{
  //clear out all the registers
  rtc.initClock();
  rtc.setDate(_day, _week, _month, 0, _year);
  rtc.setTime(_hour, _minute, _sec);
}

String getRtcTimeString() {
  dateStr = rtc.formatDate();
  dateStr += " ";
  dateStr += rtc.formatTime();

  return dateStr;
}
